/*
  HI65 - a high-level Commodore 65 emulator
  Copyright (C) 2013  Simone Gremmo
  Contact: devilmaster@email.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  http://www.gnu.org/licenses/gpl.txt
*/

#include <allegro.h>
#include <stdio.h>
#include "f8x8.h"
#include "gdef.h"
#include "pro.h"
#include "var.h"

uchar load(uchar *str)
{
 /* uint i,j,k,q;
 int c; */
 uint i;
 size_t actual_size;
 FILE *pFile;
 
 // The whole program memory is set to 0 just before loading the program
 for (i=0; i<PROGRAMSIZE; i++)
  program[i]=0;
 
 pFile=fopen ((const char *) str,"rb"); // reads from a binary file
 if (!pFile)
  return 1;
 else
 {
  actual_size=fread((void*)program,sizeof(unsigned char),PROGRAMSIZE,pFile);
  fclose (pFile);
  return 0;
 }
}

void fprints(uchar *str)
{
 uchar c;
 
 FILE *pFile;
 
 pFile = fopen ("c65log.txt","a");
 while ((c=*str++)!=0)
  fputc (c,pFile);
 fclose (pFile);
}

void fEnter(void)
{
 FILE *pFile;
 
 pFile = fopen ("c65log.txt","a");
 fputc(13,pFile);
 fputc(10,pFile);
 fclose (pFile);
}

void fprintn(uint number) // it saves numbers up to 65535 to a file
{
 uchar asciinumber[5];
 uchar i,startingdigit;
 uint numberaux;
 
 FILE *pFile;
 
 for (i=0; i<5; i++)
  asciinumber[i]=' ';
 
 if (number<10)
 {
  startingdigit=4;
  asciinumber[4]=(uchar)(number-'0');
 }
 else if (number<100)
 {
  // ex. if number is 12
  startingdigit=3;
  numberaux=number/10; // 12/10=1
  asciinumber[3]=(uchar)(numberaux-'0');
  numberaux=number-numberaux*10; //12-1*10=2
  asciinumber[4]=(uchar)(numberaux-'0');
 }
 else if (number<1000)
 {
  // ex. if number is 123
  startingdigit=2;
  numberaux=number/100; // 123/100=1
  asciinumber[2]=(uchar)(numberaux-'0');
  numberaux=number-numberaux*100; // 123-1*100=23
  numberaux=numberaux/10; // 23/10=2
  asciinumber[3]=(uchar)(numberaux-'0');
  numberaux=number/10; // 123/10=12
  numberaux=number-numberaux*10; // 123-12*10=3
  asciinumber[4]=(uchar)(numberaux-'0');
 }
 else if (number<10000)
 {
  // ex. if number is 1234
  startingdigit=1;
  numberaux=number/1000; // 1234/1000=1
  asciinumber[1]=(uchar)(numberaux-'0');
  numberaux=number-numberaux*1000; // 1234-1*1000=234
  numberaux=numberaux/100; // 234/100=2
  asciinumber[2]=(uchar)(numberaux-'0');
  numberaux=number/10; // 1234/10=123
  numberaux=numberaux-((numberaux/10)*10); // 123-12*10=3
  asciinumber[3]=(uchar)(numberaux-'0');
  numberaux=number/10; // 1234/10=123
  numberaux=number-numberaux*10; // 1234-1230=4
  asciinumber[4]=(uchar)(numberaux-'0');
 }
 else
 {
  // ex. if number is 12345
  startingdigit=0;
  numberaux=number/10000; // 12345/10000=1
  asciinumber[0]=(uchar)(numberaux-'0');
  numberaux=number-numberaux*10000; // 12345-1*10000=2345
  numberaux/=1000; // 2345/1000=2
  asciinumber[1]=(uchar)(numberaux-'0');
  numberaux=number/100; // 12345/100=123
  numberaux=numberaux-((numberaux/10)*10); // 123-(12*10)=3
  asciinumber[2]=(uchar)(numberaux-'0');
  numberaux=number/10; // 12345/10=1234
  numberaux=numberaux-((numberaux/10)*10); // 1234-(123*10)=4
  asciinumber[3]=(uchar)(numberaux-'0');
  numberaux=number-((number/10)*10); // 12345-(1234*10)=5
  asciinumber[4]=(uchar)(numberaux-'0');
 }
 pFile = fopen ("c65log.txt","a");
 for (i=0; i<5; i++)
 {
  if (asciinumber[i]!=' ')
   asciinumber[i]-=160;
  fputc (asciinumber[i],pFile);
 }
 fclose (pFile);
}

void setresolution(void)
{
 uchar cfgarray[1];
 size_t actual_size;
 FILE *pFile;
 
 pFile=fopen ("hi65.cfg","rb");
 actual_size=fread((void*)cfgarray,sizeof(unsigned char),1,pFile);
 fclose (pFile);
 
 switch(cfgarray[0])
 {
  // NOTE: resolutions like 320x200 are useless, because the C65 line
  // mode has an X resolution of 640 pixels. For the same reason, a
  // resolution of 1024x768 is not included, because 1024 is not a
  // multiple of 640
  case 0: screen_width=640;
          screen_height=400;
          screen_border_x=screen_border_y=0;
          break;
  case 1: screen_width=640;
          screen_height=480;
          screen_border_x=0;
          screen_border_y=40;
          break;
  case 2: screen_width=800;
          screen_height=600;
          screen_border_x=80;
          screen_border_y=60;
          break;
  case 3: screen_width=1280;
          screen_height=720;
          screen_border_x=screen_border_y=0;
          break;
  case 4: screen_width=1280;
          screen_height=800;
          screen_border_x=screen_border_y=0;
          break;
 }
}


