/*
  HI65 - a high-level Commodore 65 emulator
  Copyright (C) 2013  Simone Gremmo
  Contact: devilmaster@email.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  http://www.gnu.org/licenses/gpl.txt
*/

#define MAIN

#include <allegro.h>
#include <stdio.h>
#include <iostream>
#include "f8x8.h"
#include "gdef.h"
#include "pro.h"
#include "var.h"
 
int main(int argc, char* argv[])
{
 uchar i;
 
 // We expect 2 arguments: the name of the executable and the name of
 // the program to load
 
 // you should always do this at the start of Allegro programs
 if (allegro_init())
  return 1;

 //set up timer handler
 install_timer();
 
 // set up the keyboard handler
 install_keyboard(); 
 
 shifted=0;
 
 // configure the resolution
 setresolution();
 
 if (startscreen(0))
  return 1;
 else
  Ready();
  
 initvars(); // the variables are initialized
 
 if (argc>=2)
 {
  loaderror=0;
  loaderror=load((uchar *)argv[1]); // the program is loaded
  
  // This simulates what (probably) happens when you load a BASIC
  // program on a real C65
  simulateloadinstructions((uchar *)argv[1],1);
  
  if (!loaderror)
  {
   prints((uchar *)"run");
   Enter();
   program_loaded_from_emulator=0;
   Run(1); // WE RUN THE PROGRAM!!!
  }
 
  // If two consecutive cycles have the same instruction end address,
  // the program counter is stuck (like in a 10 GOTO 10 instruction)
#ifdef DEBUG_INSTRUCTIONS
  if (instruction_end_address==old_instruction_end_address)
  {
   Enter();
   forecolor=makecol(240,0,0);
   prints((uchar *)"program counter stuck in loop"); Enter();
   prints((uchar *)"address "); printn(pc); prints((uchar *)" value "); printn(program[pc]);
   forecolor=makecol(240,240,240);
  }
#endif
  Ready();
 }
 else
  program_loaded_from_emulator=1;
 
 // This may look like the main loop, where most of the emulation is
 // done, but it's not. The main loop is actually inside the Run
 // function in interpreter.cpp, where the Interpret function is called
 // once per iteration. That is what runs the programs.
 // This loop handles the keyboard when the emulator is NOT running a
 // program, and the interpretation of direct commands is triggered
 // by pressing the Return key.
 while(1)
 {
  if (keyboard(0)) // quit if ESC is pressed
   return 0;
 }

 return 0;
}

END_OF_MAIN()

